@@ -5,7 +5,7 @@
  *  2005 - 2006 Peter Selinger <selinger@users.sourceforge.net>
  *
  *  Note: this subdriver was initially generated as a "stub" by the
- *  path-to-subdriver script. It must be customized.
+ *  gen-usbhid-subdriver script. It must be customized.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@
  */
 static double	battery_scale = 1;
 
-static void *cps_battery_scale(void)
+static void *cps_battery_scale(USBDevice_t *device)
 {
 	battery_scale = 0.667;
 	return NULL;
@@ -193,8 +193,7 @@ static const char *cps_format_serial(HIDDevice_t *hd) {
  * the device is supported by this subdriver, else 0. */
 static int cps_claim(HIDDevice_t *hd) {
 
-	int status = is_usb_device_supported(cps_usb_device_table, hd->VendorID,
-								 hd->ProductID);
+	int status = is_usb_device_supported(cps_usb_device_table, hd);
 
 	switch (status) {
 
