@@ -32,14 +32,14 @@ char *getval(const char *var);
 int testvar(const char *var);
 
 /* extended variable table - used for -x defines/flags */
-typedef struct {
+typedef struct vartab_s {
 	int	vartype;	/* VAR_* value, below			 */
 	char	*var;		/* left side of =, or whole word if none */
 	char	*val;		/* right side of = 			 */
 	char	*desc;		/* 40 character description for -h text	 */
 	int	found;		/* set once encountered, for testvar()	 */
-	void	*next;
-}	vartab_t;
+	struct vartab_s	*next;
+} vartab_t;
 
 /* flags to define types in the vartab */
 
