@@ -56,9 +56,6 @@ static void get_upsdesc(nut_ctype_t *client, const char *upsname)
 		return;
 	}
 
-	if (!ups_available(ups, client))
-		return;
-
 	if (ups->desc) {
 		pconf_encode(ups->desc, esc, sizeof(esc));
 		sendback(client, "UPSDESC %s \"%s\"\n", upsname, esc);
@@ -149,6 +146,11 @@ static void get_type(nut_ctype_t *client, const char *upsname, const char *var)
 		return;
 	}
 
+	if (node->range_list) {
+		sendback(client, "%s RANGE\n", buf);
+		return;
+	}
+
 	if (node->flags & ST_FLAG_STRING) {
 		sendback(client, "%s STRING:%d\n", buf, node->aux);
 		return;
