@@ -18,10 +18,11 @@
  */
 
 #include "common.h"
+#include <ltdl.h>
 
 int nutscan_avail_avahi = 0;
 int nutscan_avail_ipmi = 0;
-int nutscan_avail_nut = 1;
+int nutscan_avail_nut = 0;
 int nutscan_avail_snmp = 0;
 int nutscan_avail_usb = 0;
 int nutscan_avail_xml_http = 0;
@@ -31,6 +32,7 @@ int nutscan_load_snmp_library(void);
 int nutscan_load_neon_library(void);
 int nutscan_load_avahi_library(void);
 int nutscan_load_ipmi_library(void);
+int nutscan_load_upsclient_library(void);
 
 void nutscan_init(void)
 {
@@ -49,4 +51,27 @@ void nutscan_init(void)
 #ifdef WITH_FREEIPMI
 	nutscan_avail_ipmi = nutscan_load_ipmi_library();
 #endif
+	nutscan_avail_nut = nutscan_load_upsclient_library();
+}
+
+void nutscan_free(void)
+{
+	if( nutscan_avail_usb ) {
+		lt_dlexit();
+	}
+	if( nutscan_avail_snmp ) {
+		lt_dlexit();
+	}
+	if( nutscan_avail_xml_http ) {
+		lt_dlexit();
+	}
+	if( nutscan_avail_avahi ) {
+		lt_dlexit();
+	}
+	if( nutscan_avail_ipmi ) {
+		lt_dlexit();
+	}
+	if( nutscan_avail_nut ) {
+		lt_dlexit();
+	}
 }
