@@ -50,6 +50,8 @@ static usb_device_id_t usb_device_table[] = {
 	{ 0x0463, 0x0001, "usbhid-ups" },
 	{ 0x0463, 0xffff, "usbhid-ups" },
 	{ 0x047c, 0xffff, "usbhid-ups" },
+	{ 0x04b4, 0x5500, "riello_usb" },
+	{ 0x04d8, 0xd004, "usbhid-ups" },
 	{ 0x050d, 0x0375, "usbhid-ups" },
 	{ 0x050d, 0x0551, "usbhid-ups" },
 	{ 0x050d, 0x0750, "usbhid-ups" },
@@ -100,6 +102,7 @@ static usb_device_id_t usb_device_table[] = {
 	{ 0x09ae, 0x3013, "usbhid-ups" },
 	{ 0x09ae, 0x3014, "usbhid-ups" },
 	{ 0x09ae, 0x3015, "usbhid-ups" },
+	{ 0x09ae, 0x3016, "usbhid-ups" },
 	{ 0x09ae, 0x4001, "usbhid-ups" },
 	{ 0x09ae, 0x4002, "usbhid-ups" },
 	{ 0x09ae, 0x4003, "usbhid-ups" },
