@@ -297,6 +297,12 @@ sub find_usbdevs
 			elsif ($nameFile eq "richcomm_usb.c") {
 				$driver="richcomm_usb";
 			}
+			elsif ($nameFile eq "riello_usb.c") {
+				$driver="riello_usb";
+			}
+			elsif ($nameFile eq "nutdrv_qx.c") {
+				$driver="nutdrv_qx";
+			}
 			else {
 				die "Unknown driver type: $nameFile";
 			}
